from __future__ import annotations

from typing import Any, Literal, TypedDict, Unpack

import pulse as ps

from ..box import BoxProps
from ..styles import StyleFn
from ..types import MantineSize

ContainerStylesNames = Literal["root"]
ContainerAttributes = dict[ContainerStylesNames, dict[str, Any]]
ContainerStyles = dict[ContainerStylesNames, ps.CSSProperties]
ContainerClassNames = dict[ContainerStylesNames, str]


class ContainerCSSVariables(TypedDict, total=False):
	root: dict[Literal["--container-size"], str]


class ContainerProps(ps.HTMLDivProps, BoxProps, total=False):  # pyright: ignore[reportIncompatibleVariableOverride]
	size: MantineSize | str | float
	"""`max-width` of the container, value is not responsive – it is the same
    for all screen sizes. Numbers are converted to rem. Ignored when `fluid`
    prop is set. @default `'md'`"""
	fluid: bool
	"""If set, the container takes 100% width of its parent and `size` prop is
    ignored. @default `false`"""
	strategy: Literal["block", "grid"]
	"""Centering strategy @default `'block'`"""

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	variant: str
	"""Component variant, if applicable"""
	classNames: ContainerClassNames | StyleFn[ContainerProps, Any, ContainerClassNames]
	"""Additional class names passed to elements"""
	styles: ContainerStyles | StyleFn[ContainerProps, Any, ContainerStyles]
	"""Additional styles passed to elements"""
	vars: ContainerCSSVariables
	"""CSS variables passed to elements"""
	attributes: ContainerAttributes
	"""Additional attributes passed to elements"""


@ps.react_component(ps.Import("Container", "@mantine/core"))
def Container(
	*children: ps.Node, key: str | None = None, **props: Unpack[ContainerProps]
): ...



from typing import Any

import pulse as ps

_Pill = ps.Import("Pill", "@mantine/core")


@ps.react_component(_Pill)
def Pill(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_Pill.Group)
def PillGroup(*children: ps.Node, key: str | None = None, **props: Any): ...



from __future__ import annotations

from typing import Unpack

import pulse as ps

from pulse_mantine.core.box import BoxProps


class SpaceProps(ps.HTMLDivProps, BoxProps, total=False):  # pyright: ignore[reportIncompatibleVariableOverride]
	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	variant: str
	"""Component variant, if applicable"""

	# no classNames, styles, vars, attributes


@ps.react_component(ps.Import("Space", "@mantine/core"))
def Space(*children: ps.Node, key: str | None = None, **props: Unpack[SpaceProps]): ...



from __future__ import annotations

from typing import Any, Literal, TypedDict, Unpack

import pulse as ps

from ..box import BoxProps
from ..styles import StyleFn

AspectRatioStylesNames = Literal["root"]
AspectRatioAttributes = dict[AspectRatioStylesNames, dict[str, Any]]
AspectRatioStyles = dict[AspectRatioStylesNames, ps.CSSProperties]
AspectRatioClassNames = dict[AspectRatioStylesNames, str]


class AspectRatioCSSVariables(TypedDict, total=False):
	root: dict[Literal["--ar-ratio"], str]


class AspectRatioProps(ps.HTMLDivProps, BoxProps, total=False):  # pyright: ignore[reportIncompatibleVariableOverride]
	ratio: float
	"Aspect ratio, for example, `16 / 9`, `4 / 3`, `1920 / 1080` @default `1`"

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	variant: str
	"""Component variant, if applicable"""
	classNames: (
		AspectRatioClassNames | StyleFn[AspectRatioProps, Any, AspectRatioClassNames]
	)
	"""Additional class names passed to elements"""
	styles: AspectRatioStyles | StyleFn[AspectRatioProps, Any, AspectRatioStyles]
	"""Additional styles passed to elements"""
	vars: StyleFn[AspectRatioProps, Any, AspectRatioCSSVariables]
	"""CSS variables for the component"""
	attributes: AspectRatioAttributes
	"""Additional attributes passed to elements"""


@ps.react_component(ps.Import("AspectRatio", "@mantine/core"))
def AspectRatio(
	*children: ps.Node, key: str | None = None, **props: Unpack[AspectRatioProps]
): ...



from typing import Any

import pulse as ps

_FocusTrap = ps.Import("FocusTrap", "@mantine/core")


@ps.react_component(_FocusTrap)
def FocusTrap(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_FocusTrap.InitialFocus)
def FocusTrapInitialFocus(key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps

_Input = ps.Import("Input", "@mantine/core")


@ps.react_component(_Input)
def Input(key: str | None = None, **props: Any): ...


@ps.react_component(_Input.Label)
def InputLabel(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_Input.Error)
def InputError(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_Input.Description)
def InputDescription(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_Input.Placeholder)
def InputPlaceholder(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_Input.Wrapper)
def InputWrapper(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_Input.ClearButton)
def InputClearButton(key: str | None = None, **props: Any): ...



from typing import Any

import pulse as ps

_ScrollArea = ps.Import("ScrollArea", "@mantine/core")


@ps.react_component(_ScrollArea)
def ScrollArea(*children: ps.Node, key: str | None = None, **props: Any): ...


@ps.react_component(_ScrollArea.Autosize)
def ScrollAreaAutosize(*children: ps.Node, key: str | None = None, **props: Any): ...



from __future__ import annotations

import uuid
from collections.abc import Callable, Iterable, Mapping
from typing import Any, Literal, NotRequired, Required, Self, TypedDict, Unpack, cast
from uuid import uuid4

import pulse as ps

from ..box import BoxProps
from ..styles import StyleFn
from ..theme import MantineRadius
from ..types import MantineColor

ps.Import("@mantine/notifications/styles.css", side_effect=True)


NotificationRootCSSVariables = Literal["--notification-radius", "--notification-color"]
NotificationStylesNames = Literal[
	"root",
	"icon",
	"loader",
	"body",
	"title",
	"description",
	"closeButton",
]
NotificationAttributes = dict[NotificationStylesNames, dict[str, Any]]
NotificationStyles = dict[NotificationStylesNames, ps.CSSProperties]
NotificationClassNames = dict[NotificationStylesNames, str]


class NotificationCssVariables(TypedDict, total=False):
	root: dict[NotificationRootCSSVariables, str]


class NotificationProps(  # pyright: ignore[reportIncompatibleVariableOverride]
	ps.HTMLDivProps,
	BoxProps,
	total=False,
):
	variant: str
	"""Component variant"""
	onClose: ps.EventHandler0
	"""Called when the close button is clicked"""
	color: MantineColor
	"""Controls notification line or icon color"""
	radius: MantineRadius
	"""Controls notification border radius"""
	icon: ps.Element
	"""Notification icon, replaces color line"""
	title: ps.Element  # pyright: ignore[reportIncompatibleVariableOverride]
	"""Notification title, displayed above the message body"""
	loading: bool
	"""If set, the Loader component is displayed instead of the icon"""
	withBorder: bool
	"""Adds border to the root element"""
	withCloseButton: bool
	"""If set, the close button is visible"""
	closeButtonProps: Mapping[str, Any]
	"""Props passed down to the close button"""
	loaderProps: Mapping[str, Any]
	"""Props passed down to the Loader component"""

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	classNames: (
		NotificationClassNames | StyleFn[NotificationProps, Any, NotificationClassNames]
	)
	"""Additional class names passed to elements"""
	styles: NotificationStyles | StyleFn[NotificationProps, Any, NotificationStyles]
	"""Additional styles passed to elements"""
	vars: StyleFn[NotificationProps, Any, NotificationCssVariables]
	"""CSS variables for the component"""
	attributes: NotificationAttributes
	"""Additional attributes passed to elements"""


@ps.react_component(ps.Import("Notification", "@mantine/core"))
def Notification(
	*children: ps.Node, key: str | None = None, **props: Unpack[NotificationProps]
): ...


NotificationsStylesNames = Literal["root", "notification"]
NotificationsAttributes = dict[NotificationsStylesNames, dict[str, Any]]
NotificationsStyles = dict[NotificationsStylesNames, ps.CSSProperties]
NotificationsClassNames = dict[NotificationsStylesNames, str]


class NotificationsCssVariables(TypedDict, total=False):
	root: dict[
		Literal["--notifications-z-index", "--notifications-container-width"],
		str,
	]


class NotificationsProps(  # pyright: ignore[reportIncompatibleVariableOverride]
	ps.HTMLDivProps,
	BoxProps,
	total=False,
):
	position: NotificationPosition
	"""Notifications default position"""
	autoClose: int | bool
	"""Auto close timeout for all notifications in ms, `False` to disable auto close"""
	transitionDuration: int
	"""Notification transition duration in ms"""
	containerWidth: int | str
	"""Notification width, cannot exceed 100%"""
	notificationMaxHeight: int | str
	"""Notification max-height, used for transitions"""
	limit: int
	"""Maximum number of notifications displayed at a time"""
	zIndex: int | str
	"""Notifications container z-index"""
	portalProps: Mapping[str, Any]
	"""Props passed down to the Portal component"""
	withinPortal: bool
	"""Determines whether notifications container should be rendered inside Portal"""

	# Styles API props
	unstyled: bool
	"""Removes default styles from the component"""
	classNames: (
		NotificationsClassNames
		| StyleFn[NotificationsProps, Any, NotificationsClassNames]
	)
	"""Additional class names passed to elements"""
	styles: NotificationsStyles | StyleFn[NotificationsProps, Any, NotificationsStyles]
	"""Additional styles passed to elements"""
	vars: StyleFn[NotificationsProps, Any, NotificationsCssVariables]
	"""CSS variables for the component"""
	attributes: NotificationsAttributes
	"""Additional attributes passed to elements"""


NotificationPosition = Literal[
	"top-left",
	"top-right",
	"top-center",
	"bottom-left",
	"bottom-right",
	"bottom-center",
]


class NotificationDataWithoutId(NotificationProps, total=False):
	id: str
	"""Notification id, can be used to close or update notification"""
	message: Required[str]
	"""Main notification message. Real API also supports nodes, but we can't handle that yet."""
	position: NotificationPosition
	"""Notification position"""
	autoClose: int | bool
	"""Per-notification auto close timeout"""
	onClose: Callable[[Self], None]
	"""Called when notification closes"""
	onOpen: Callable[[Self], None]
	"""Called when notification opens"""


class NotificationData(NotificationDataWithoutId):
	id: str  # pyright: ignore[reportGeneralTypeIssues]
	onClose: NotRequired[Callable[[Self], None]]
	"""Called when notification closes"""
	onOpen: NotRequired[Callable[[Self], None]]
	"""Called when notification opens"""


@ps.react_component(ps.Import("Notifications", "pulse-mantine"))
def NotificationsInternal(
	*children: ps.Node,
	key: str | None = None,
	channelId: str | None = None,
	**props: Unpack[NotificationsProps],
): ...


NOTIFICATIONS_CHANNEL_ID = uuid.uuid4().hex


class NotificationsStore(ps.State):
	_channel: ps.Channel
	registry: dict[str, NotificationData]
	visible_ids: list[str]
	queued_ids: list[str]

	def __init__(self) -> None:
		self._channel = ps.channel(NOTIFICATIONS_CHANNEL_ID)
		self.registry = {}
		self.visible_ids = []
		self.queued_ids = []
		self._channel.on("stateSync", self._on_state_sync)

	def show(self, kwargs: NotificationDataWithoutId) -> str:
		ident, payload = ensure_id(kwargs)
		existing = self.registry.get(ident)
		merged = payload if not existing else existing | payload
		self.registry[ident] = merged
		self._channel.emit("show", serialize(merged))
		return ident

	def update(self, kwargs: NotificationData) -> str:
		ident = kwargs["id"]
		payload = kwargs
		existing = self.registry.get(ident)
		merged = payload if not existing else existing | payload
		self.registry[ident] = merged
		self._channel.emit("update", serialize(merged))
		return ident

	def hide(self, id: str) -> str:
		self._channel.emit("hide", {"id": id})
		return id

	def clean(self) -> None:
		self._channel.emit("clean")

	def cleanQueue(self) -> None:
		self._channel.emit("cleanQueue")

	def getVisible(self) -> list[NotificationData]:
		return [self.registry[id_] for id_ in self.visible_ids]

	def getQueued(self) -> list[NotificationData]:
		return [self.registry[id_] for id_ in self.queued_ids]

	def getState(self) -> list[NotificationData]:
		return self.getVisible() + self.getQueued()

	def isVisible(self, id: str) -> bool:
		return id in self.visible_ids

	def isQueued(self, id: str) -> bool:
		return id in self.queued_ids

	def updateState(
		self,
		update: Callable[[list[NotificationData]], Iterable[NotificationData]]
		| Iterable[NotificationData],
	) -> None:
		current = self.getState()
		if callable(update):
			result = update(current)
		else:
			result = update
		new_notifications: list[NotificationData] = []
		for item in result:
			ident, item = ensure_id(item)
			self.registry[ident] = item
			new_notifications.append(item)
		self._channel.emit(
			"updateState",
			{"notifications": [serialize(x) for x in new_notifications]},
		)

	def _on_state_sync(self, payload: Any) -> None:
		if not isinstance(payload, dict):
			return
		visible = cast(list[str], payload["notifications"])
		queue = cast(list[str], payload["queue"])

		# Create sets once to avoid repeated conversions
		current_visible = set(visible)
		current_queue = set(queue)
		current_ids = current_visible | current_queue
		registry_keys = set(self.registry.keys())
		previous_visible = set(self.visible_ids)
		previous_queue = set(self.queued_ids)
		previous_ids = previous_visible | previous_queue

		# Error if we receive IDs that are not in the registry
		unknown_ids = current_ids - registry_keys
		if unknown_ids:
			raise ValueError(f"Received unknown notification IDs: {unknown_ids}")

		# Determine newly opened and closed IDs
		newly_opened = current_visible - previous_visible
		closed_ids = previous_ids - current_ids

		# Call onOpen handlers for newly opened IDs (only when they appear in visible)
		for ident in newly_opened:
			notification = self.registry[ident]
			if "onOpen" in notification and callable(notification["onOpen"]):
				notification["onOpen"](notification)

		# Call onClose handlers for closed IDs
		for ident in closed_ids:
			notification = self.registry[ident]
			if "onClose" in notification and callable(notification["onClose"]):
				notification["onClose"](notification)
			# Remove closed notifications from the registry
			self.registry.pop(ident, None)

		# Update the current state
		self.visible_ids = visible
		self.queued_ids = queue


def Notifications(
	*children: ps.Node,
	key: str | None = None,
	**props: Unpack[NotificationsProps],
):
	return NotificationsInternal(
		*children,
		key=key,
		**props,
		channelId=NOTIFICATIONS_CHANNEL_ID,
	)


def ensure_id(
	value: NotificationDataWithoutId | NotificationData,
) -> tuple[str, NotificationData]:
	if not isinstance(value, Mapping):
		raise TypeError("Notification payload must be a mapping")
	ident = value.get("id")
	if not ident:
		ident = uuid4().hex
		value["id"] = ident
	return ident, cast(NotificationData, value)


def serialize(value: NotificationData):
	return {k: v for k, v in value.items() if k not in ("onClose", "onOpen")}


notifications_state = ps.global_state(NotificationsStore)


class NotificationsApi:
	__slots__: tuple[()] = ()

	def _resolve(self) -> NotificationsStore:
		return notifications_state()

	def show(self, **payload: Unpack[NotificationDataWithoutId]):
		return self._resolve().show(payload)

	def update(self, **payload: Unpack[NotificationData]):
		return self._resolve().update(payload)

	def hide(self, id: str):
		return self._resolve().hide(id)

	def clean(self) -> None:
		self._resolve().clean()

	def cleanQueue(self) -> None:
		self._resolve().cleanQueue()

	def updateState(
		self,
		update: Callable[[list[NotificationData]], Iterable[NotificationData]]
		| Iterable[NotificationData],
	) -> None:
		self._resolve().updateState(update)

	def getVisible(self):
		return self._resolve().getVisible()

	def getQueued(self):
		return self._resolve().getQueued()

	def getState(self):
		return self._resolve().getState()

	def isVisible(self, id: str) -> bool:
		return self._resolve().isVisible(id)

	def isQueued(self, id: str) -> bool:
		return self._resolve().isQueued(id)


notifications = NotificationsApi()



from __future__ import annotations

from typing import Any, Literal, TypedDict, Unpack

import pulse as ps

from .styles import CSSVariables
from .theme import MantineTheme, MantineThemeOverride
from .types import MantineColorScheme

MantineProviderForceColorScheme = Literal["light", "dark"]
MantineProviderEnvironment = Literal["default", "test"]


class MantineProviderProps(TypedDict, total=False):
	"""Theme override object"""

	theme: MantineThemeOverride
	colorSchemeManager: MantineColorSchemeManager
	"""Used to retrieve/set color scheme value in external storage, by default uses `window.localStorage`"""
	defaultColorScheme: MantineColorScheme
	"""Default color scheme value used when `colorSchemeManager` cannot retrieve value from external storage, `light` by default"""
	forceColorScheme: MantineProviderForceColorScheme
	"""Forces color scheme value, if set, MantineProvider ignores `colorSchemeManager` and `defaultColorScheme`"""
	cssVariablesSelector: str
	"""CSS selector to which CSS variables should be added, `:root` by default"""
	withCssVariables: bool
	"""Determines whether theme CSS variables should be added to given `cssVariablesSelector` @default `true`"""
	deduplicateCssVariables: bool
	"""Determines whether CSS variables should be deduplicated: if CSS variable has the same value as in default theme, it is not added in the runtime. @default `true`."""
	getRootElement: ps.JsFunction[ps.HTMLElement | None]
	"""Function to resolve root element to set `data-mantine-color-scheme` attribute, must return undefined on server, `() => document.documentElement` by default"""
	classNamesPrefix: str
	"""A prefix for components static classes (for example {selector}-Text-root), `mantine` by default"""
	getStyleNonce: ps.JsFunction[str]
	"""Function to generate nonce attribute added to all generated `<style />` tags"""
	cssVariablesResolver: CSSVariablesResolver
	"""Function to generate CSS variables based on theme object"""
	withStaticClasses: bool
	"""Determines whether components should have static classes, for example, `mantine-Button-root`. @default `true`"""
	withGlobalClasses: bool
	"""Determines whether global classes should be added with `<style />` tag. Global classes are required for `hiddenFrom`/`visibleFrom` and `lightHidden`/`darkHidden` props to work. @default `true`."""
	stylesTransform: MantineStylesTransform
	"""An object to transform `styles` and `sx` props into css classes, can be used with CSS-in-JS libraries"""
	env: MantineProviderEnvironment
	"""Environment at which the provider is used, `'test'` environment disables all transitions and portals"""


@ps.react_component(
	ps.Import("MantineProvider", "@mantine/core"),
)
def MantineProvider(*children: ps.Node, key: str | None = None, **props: Any): ...


class MantineStylesTransform(TypedDict, total=False):
	sx: ps.JsFunction[ps.JsFunction[Any, str]]
	styles: ps.JsFunction[ps.JsFunction[Any, Any, dict[str, str]]]


class ConvertCSSVariablesInput(TypedDict):
	variables: CSSVariables
	"Shared CSS variables that should be accessible independent from color scheme"
	dark: CSSVariables
	"CSS variables available only in dark color scheme"
	light: CSSVariables
	"CSS variables available only in light color scheme"


CSSVariablesResolver = ps.JsFunction[MantineTheme, ConvertCSSVariablesInput]


class MantineColorSchemeManager(TypedDict):
	get: ps.JsFunction[MantineColorScheme, MantineColorScheme]
	"""Function to retrieve color scheme value from external storage, for example window.localStorage
    
    JS: get: (defaultValue: MantineColorScheme) => MantineColorScheme"""

	set: ps.JsFunction[MantineColorScheme, None]
	"""Function to set color scheme value in external storage, for example window.localStorage
    
    JS: set: (value: MantineColorScheme) => void"""

	subscribe: ps.JsFunction[ps.JsFunction[MantineColorScheme, None], None]
	"""Function to subscribe to color scheme changes triggered by external events
    
    JS: subscribe: (onUpdate: (colorScheme: MantineColorScheme) => void) => void"""

	unsubscribe: ps.JsFunction[None]
	"""Function to unsubscribe from color scheme changes triggered by external events
    
    JS: unsubscribe: () => void"""

	clear: ps.JsFunction[None]
	"""Function to clear value from external storage
    
    JS: clear: () => void"""


class HeadlessMantineProviderProps(TypedDict, total=False):
	theme: MantineThemeOverride
	"""Theme override object"""

	env: MantineProviderEnvironment
	"""Environment at which the provider is used, 'test' environment disables all transitions and portals"""


@ps.react_component(ps.Import("HeadlessMantineProvider", "@mantine/core"))
def HeadlessMantineProvider(
	*children: ps.Node,
	key: str | None = None,
	**props: Unpack[HeadlessMantineProviderProps],
): ...
